import logging

from junitparser import JUnitXml, Failure
from tabulate import tabulate

THRESHOLD = 0.02
SLOW_ERROR_MSG = 'Test duration exceeded threshold'
FAIL_TEXT = 'FAIL'


def report_slow_tests(junit_file_path: str, xml_output_file_name: str, display_results: bool):
    """
    Takes a junit test results file and determines if there are any slow tests

    Args:
        junit_file_path (str) : The file path of the junit file results
        xml_output_file_name (str) : Output file name, if a junit file is to be produced. Do not include the file
                                    extension
        display_results (str) : Displays test results if set to true
    """
    junit_xml = JUnitXml.fromfile(junit_file_path)

    test_results, xml_output = parse_test_results(junit_xml)

    if display_results:
        print(tabulate(test_results, headers=["Test", "Time (s)", "Result", "Message"]))

    if xml_output_file_name:
        xml_output.write(xml_output_file_name+'.xml')
        logging.info('junit file created: %s.xml', xml_output_file_name)


def parse_test_results(junit_xml: JUnitXml):
    test_results = []

    """
    IF TEST FAILED => FAILED
    IF TEST TOO SLOW => FAILED
    IF TEST TOO SLOW + FAILED => FAILED
    OTHERWISE => PASS
    """

    for testcase in junit_xml:
        if testcase.time > THRESHOLD:
            result = FAIL_TEXT
            if testcase.result and testcase.result[0]:
                testcase.result[0].message += ' ' + SLOW_ERROR_MSG
            else:
                testcase.result = [Failure(SLOW_ERROR_MSG)]
            test_results.append([testcase.name, testcase.time, result, testcase.result[0].message])
        elif testcase.result and testcase.result[0]:
            result = FAIL_TEXT
            test_results.append([testcase.name, testcase.time, result, testcase.result[0].message])
        else:
            result = 'PASS'
            test_results.append([testcase.name, testcase.time, result])

    return test_results, junit_xml



import pytest
from junitparser import TestCase, TestSuite, Error


# Prevent pytest from trying to collect junitparser test objects as tests:
TestCase.__test__ = False
TestSuite.__test__ = False

from slowtestreporter import slowtestreporter
from slowtestreporter.slowtestreporter import report_slow_tests


def test_should_throw_exception_when_file_is_not_found():
    with pytest.raises(FileNotFoundError):
        report_slow_tests('filedoesnotexist.xml', 'my_results', False)


def test_should_throw_exception_when_no_file_provided():
    with pytest.raises(FileNotFoundError):
        report_slow_tests('', 'results', False)


def test_should_not_change_results_when_single_test_is_fast():
    case1 = TestCase('case1', 'Test', 0.01)
    case1.result = []
    suite = TestSuite('suite1')
    suite.add_testcase(case1)

    test_results, junit_xml = slowtestreporter.parse_test_results(suite)
    assert slowtestreporter.SLOW_ERROR_MSG not in test_results[0], 'Expected no slow test error'


def test_should_not_change_results_when_single_failed_test_is_fast():
    case1 = TestCase('case1', 'FailedTest', 0.01)
    case1.result = [Error('Error', 'Some error type')]
    suite = TestSuite('suite1')
    suite.add_testcase(case1)

    test_results, junit_xml = slowtestreporter.parse_test_results(suite)
    assert slowtestreporter.SLOW_ERROR_MSG not in test_results[0], 'Expected no slow test error despite failed test'


def test_should_keep_test_failed_when_single_failed_test_is_fast():
    case1 = TestCase('case1', 'FailedTest', 0.01)
    case1.result = [Error('Error', 'Some error type')]
    suite = TestSuite('suite1')
    suite.add_testcase(case1)

    test_results, junit_xml = slowtestreporter.parse_test_results(suite)
    assert slowtestreporter.FAIL_TEXT in test_results[0][2], 'Expected failed test to stay failed'


def test_should_report_slow_test_when_single_failed_test_is_slow():
    case1 = TestCase('case1', 'FailedTest', 7000)
    case1.result = [Error('Error', 'Some error type')]
    suite = TestSuite('suite1')
    suite.add_testcase(case1)

    test_results, junit_xml = slowtestreporter.parse_test_results(suite)
    assert slowtestreporter.FAIL_TEXT in test_results[0][2], 'Expected slow test error for failed and slow test'



#!/usr/bin/env python
import argparse
import logging
import sys

from slowtestreporter.slowtestreporter import report_slow_tests

DEFAULT_THRESHOLD = 0.5


def main():
    parser = argparse.ArgumentParser(description='Reports slow tests based on the junit test input.')
    parser.add_argument('-t', '--threshold', type=float, help='A float value threshold (in seconds) of when a '
                                                              'test should fail when it exceeds this value',
                        action=ThresholdArgValidationAction)
    parser.add_argument('path', type=str, help='Junit file path')
    parser.add_argument('-o', '--output-filename', help='Filename of the generated junit results without any '
                                                        'extensions. The file will be generated with a .xml extension.')
    parser.add_argument('-s', '--silent', help='Suppresses output to display', action='store_true')
    args = parser.parse_args()

    if args.silent:
        logging.basicConfig(level=logging.ERROR)
    else:
        logging.basicConfig(level=logging.INFO)
        print('JUNIT SLOW TEST REPORTER')
        print('========================')

    if not args.threshold:
        threshold = DEFAULT_THRESHOLD
        logging.info('Threshold not set, using default threshold: %ss', str(threshold))
    else:
        threshold = args.threshold
        logging.info('Threshold for slow tests: %ss', str(threshold))

    try:
        report_slow_tests(args.path, args.output_filename, not args.silent)
    except FileNotFoundError:
        logging.error('Junit file input not found.')


class ThresholdArgValidationAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        if values <= 0:
            raise ValueError("Threshold cannot be less than or equal to zero.")
        setattr(namespace, self.dest, values)


if __name__ == '__main__':
    main()
    sys.exit()
